from sqlalchemy import engine_from_config
from sqlalchemy.orm import sessionmaker
import configparser,os

import logging,logging.config
l = logging.getLogger(__name__)


def configFromConfigPath(config_uri,initlogging=True):
  def getConfigCascade(here,config_uri):
    uricascade = []
    if not os.path.isfile(config_uri):
      raise ValueError("Config file %s not found"%config_uri)    
    temp_parser = configparser.ConfigParser()
    with open(config_uri,"r") as inifile:
      temp_parser.read_string(inifile.read())
    temp_parser['DEFAULT']['here'] = here
    temp_parser["app:main"]["here"] = here      
    use_directive = temp_parser.get('app:main', 'use', fallback=None)
    if use_directive and use_directive.startswith('config:'):
      base_config_path = use_directive.split('config:', 1)[1].strip()
      uricascade += getConfigCascade(here,base_config_path)
    uricascade.append(config_uri)
    return uricascade

  config = configparser.ConfigParser() 
  here = os.path.dirname(config_uri)
  configlist = getConfigCascade(here, config_uri)
  for i in configlist:
    l.info("Loading config file %s"%i)
    with open(i,"r") as inifile:
      config.read_string(inifile.read())
  
  ##TODO: here is wrong. what should it be?
  here = os.getcwd()
  config['DEFAULT']['here'] = here
  config["app:main"]["here"] = here
  if initlogging:    
    logging.config.fileConfig(config)
  return config

def engineFromConfigPath(config_uri,initlogging=True):
  config = configFromConfigPath(config_uri,initlogging)  
  return config,engineFromConfig(config)

def engineFromConfig(config):
  engine = engine_from_config(config["app:main"], "sqlalchemy.")
  return engine

def engineFromSettings(config):
  engine = engine_from_config(config, "sqlalchemy.")
  return engine

def factoryFormSettings(config):
  engine = engineFromSettings(config)
  session_factory = sessionmaker(bind=engine)
  return session_factory


import argparse
import configparser
import sys
import logging
import jinja2

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
handler = logging.StreamHandler()
handler.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s %(levelname)-5.5s [%(name)s:%(lineno)s][%(threadName)s] %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)


def read_variable_file(filename):
    parser = configparser.ConfigParser()
    parser.read(filename)
    return parser

def parse_args(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'template',
        help='Template jinja2 file, e.g., development.ini.jinja2',
    )
    parser.add_argument(
        'variable',
        help='Variable ini file, e.g., secrets.ini',
    )
    parser.add_argument(
        '-o', '--output',
        help='Output file, default to secrets_<name of template>.ini',
    )

    return parser.parse_args(argv[1:])

def main(argv=sys.argv):
    args = parse_args(argv)
    config = read_variable_file(args.variable)

    vars_dict = {s:dict(config.items(s)) for s in config.sections()}
    logger.debug(vars_dict)

    template = jinja2.Template(open(args.template).read())
    output = template.render(vars_dict)
    if args.output is None:
        output_file = 'secrets_{}.ini'.format(args.template.split('.')[0])
    else:
        output_file = args.output
    with open(output_file, 'w') as f:
        f.write(output)
        logger.info('Output file: %s', output_file)
        
